/*
	File:		FolderMusicLibrary.java
	
	Copyright: 	 Copyright 1999-2000 Apple Computer, Inc. All rights reserved.
	
	Disclaimer:	IMPORTANT:  This Apple software is supplied to you by Apple Computer, Inc.
				("Apple") in consideration of your agreement to the following terms, and your
				use, installation, modification or redistribution of this Apple software
				constitutes acceptance of these terms.  If you do not agree with these terms,
				please do not use, install, modify or redistribute this Apple software.

				In consideration of your agreement to abide by the following terms, and subject
				to these terms, Apple grants you a personal, non-exclusive license, under Apples
				copyrights in this original Apple software (the "Apple Software"), to use,
				reproduce, modify and redistribute the Apple Software, with or without
				modifications, in source and/or binary forms; provided that if you redistribute
				the Apple Software in its entirety and without modifications, you must retain
				this notice and the following text and disclaimers in all such redistributions of
				the Apple Software.  Neither the name, trademarks, service marks or logos of
				Apple Computer, Inc. may be used to endorse or promote products derived from the
				Apple Software without specific prior written permission from Apple.  Except as
				expressly stated in this notice, no other rights or licenses, express or implied,
				are granted by Apple herein, including but not limited to any patent rights that
				may be infringed by your derivative works or by other works in which the Apple
				Software may be incorporated.

				The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
				WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
				WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
				PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
				COMBINATION WITH YOUR PRODUCTS.

				IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
				CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
				GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
				ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION
				OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
				(INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN
				ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
				
	Change History (most recent first):

*/


package com.apple.jens.radio;

import com.apple.jens.mp3.MP3File;
import java.io.*;
import java.util.Vector;


/** A MusicLibrary that makes available all the MP3 files in a directory tree.
	Directories whose names begin with "(" and "." are not scanned, however,
	which provides an easy way to "comment out" portions of the tree. (This
	behavior can be customized by overriding <code>filterDir</code>, and
	control over whether individual files are added can be achieved by
	overriding <code>filterFile</code>.)
	
	<p> The modification dates of all directories in the tree are cached,
	and checked when the <code>syncTrackList</code> method is called; if any
	directories have changed, the whole tree is re-read from scratch.
	
*/
	
public class FolderMusicLibrary implements MusicLibrary {

	public synchronized void initialize( Station station ) {
		RadioProperties props = station.getProperties();
		File dir = props.getFileProperty(kPropLibraryHome);
		if( dir==null )
			throw new IllegalArgumentException("No "+kPropLibraryHome+" property is set!");
		if( ! dir.exists() || ! dir.isDirectory() )
			throw new IllegalArgumentException(dir+" is not a directory");
		fRootDir = dir;
		Radio.LOG(1,this,"initialized on "+dir);
		buildTrackList();
	}
	

	public synchronized boolean syncTrackList( ) {
		// Rebuild the file list if the directory's been modified:
		if( hasBeenModified() ) {
			buildTrackList();
			return true;
		} else
			return false;
	}
	
	
	private boolean hasBeenModified( ) {
		int n = fDirs.size();
		for( int i=0; i<n; i++ ) {
			File f = (File)fDirs.elementAt(i);
			Long mod = (Long)fModDates.elementAt(i);
			if( f.lastModified() != mod.longValue() )
				return true;
		}
		return false;
	}
	
	
	private void buildTrackList( ) {
		fDirs.setSize(0);
		fModDates.setSize(0);
		Vector files = new Vector(100);
		fDirs.addElement(fRootDir);
		for( int i=0; i<fDirs.size(); i++ ) {
			addDir( (File)fDirs.elementAt(i), files );
		}
		int nFiles = files.size();
		fFiles = new MP3File[nFiles];
		files.copyInto(fFiles);
	}
	
	
	private void addDir( File dir, Vector files ) {
		fModDates.addElement( new Long( dir.lastModified() ) );
		String[] list = dir.list();
		for( int i=0; i<list.length; i++ ) {
			File f = new File(dir,list[i]);
			if( f.isDirectory() ) {
				if( filterDir(f) )
					fDirs.addElement(f);				// Push onto directory list
			} else {
				MP3File f3 = new MP3File(f);
				if( filterFile(f3) )
					files.addElement(f3);
				//System.out.println("FolderMusicDirector: added track "+f.getPath());
			}
		}
	}
	
	
	/** Determines whether a directory should be searched.
		By default it accepts all directories whose names don't start with "." or "(".
		You can override this to do fancier filtering.
		@return  True to accept the directory, false to skip it. */
	protected boolean filterDir( File dir ) {
		char first = dir.getName().charAt(0);
		return first!='.' && first!='(';
		//! In Java2 should check isHidden, too
	}
	
	
	/** Determines whether a file should be added to the library.
		This implementation just checks whether the filename ends with ".mp3",
		which is crude but fast.
		You can override this to filter files based on other criteria,
		possibly including the contents of ID3 tags -- but call the inherited
		method first so you don't try to parse non-MP3 files.
		@return  True to accept the file, false to skip it. */
	protected boolean filterFile( MP3File f ) {
		return f.isMP3FileType();
	}
	
	
	public synchronized MP3File[] getTrackList( ) {
		return fFiles;
	}
	
	
	public String toString( ) {
		return getClass().getName()+" on "+fRootDir;
	}
	
	
	protected File fRootDir;
	protected Vector /*[File]*/ fDirs = new Vector();
	protected Vector /*[Long]*/ fModDates = new Vector();
	protected MP3File[] fFiles;
	
}
