/*
	File:		HTMLPlaylistFilter.java
	
	Copyright: 	 Copyright 1999-2000 Apple Computer, Inc. All rights reserved.
	
	Disclaimer:	IMPORTANT:  This Apple software is supplied to you by Apple Computer, Inc.
				("Apple") in consideration of your agreement to the following terms, and your
				use, installation, modification or redistribution of this Apple software
				constitutes acceptance of these terms.  If you do not agree with these terms,
				please do not use, install, modify or redistribute this Apple software.

				In consideration of your agreement to abide by the following terms, and subject
				to these terms, Apple grants you a personal, non-exclusive license, under Apples
				copyrights in this original Apple software (the "Apple Software"), to use,
				reproduce, modify and redistribute the Apple Software, with or without
				modifications, in source and/or binary forms; provided that if you redistribute
				the Apple Software in its entirety and without modifications, you must retain
				this notice and the following text and disclaimers in all such redistributions of
				the Apple Software.  Neither the name, trademarks, service marks or logos of
				Apple Computer, Inc. may be used to endorse or promote products derived from the
				Apple Software without specific prior written permission from Apple.  Except as
				expressly stated in this notice, no other rights or licenses, express or implied,
				are granted by Apple herein, including but not limited to any patent rights that
				may be infringed by your derivative works or by other works in which the Apple
				Software may be incorporated.

				The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
				WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
				WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
				PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
				COMBINATION WITH YOUR PRODUCTS.

				IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
				CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
				GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
				ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION
				OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
				(INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN
				ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
				
	Change History (most recent first):

*/


package com.apple.jens.radio;

import com.apple.jens.mp3.*;
import com.apple.mrj.*;

import java.io.*;
import java.util.Date;


/** A MusicDirectorFilter that watches the tracks being scheduled and
	generates an HTML-format playlist file, which can be served by a web server
	running on the same machine so that clients can see what's playing.
	
	The playlist is generated from a template file using a pretty simple-minded
	variable substitution (see the <code>writeVariable</code> method for details.)
	The sample playlist file provided shows how it's used.
	
*/
	
public class HTMLPlaylistFilter implements MusicDirectorFilter {

	public static final String kPropHTMLPlaylist = "html-playlist.dst",
							   kPropHTMLPlaylistTemplate = "html-playlist.src";
	

	public void initialize( MusicIterator src, Station station ) {
		fSrc = src;
		fStation = station;
		String path = station.getProperties().getProperty(kPropHTMLPlaylist);
		if( path == null )
			Radio.WARN(1,"PlaylistHTMLDumper","No "+kPropHTMLPlaylist+" property specified!");
		else {
			fHTMLFile = new File(path);
			fTemplateFile = station.getProperties().getFileProperty(kPropHTMLPlaylistTemplate);
		}
	}
	
	
	public MP3File getNextTrack( ) {
		MP3File next = fSrc.getNextTrack();
		try{
			if( fHistory[0] == null ) {
				fHistory[1] = new MP3File(next);
				next = fSrc.getNextTrack();
			} else
				System.arraycopy(fHistory,0, fHistory,1, fHistory.length-1);
			fHistory[0] = new MP3File(next);
			next = fHistory[1];
			
			Radio.LOG(2,"PlaylistHTMLFilter",
								   "Track=" +fHistory[1].getTitle()
								+", Artist="+fHistory[1].getArtist()
								+", Album=" +fHistory[1].getAlbum() );
			
			if( fHTMLFile != null ) {
				FileOutputStream out = new FileOutputStream(fHTMLFile);
				MRJFileUtils.setFileTypeAndCreator(fHTMLFile,MRJOSType.kTypeTEXT,
														 	 new MRJOSType("MSIE"));
				try{
					if( fTemplateFile != null )
						writeTemplatePlaylist(out);
					else
						writeHardcodedPlaylist(out);
				}finally{
					out.close();
				}
			}
		}catch( Exception x ) {
			System.out.println("HTMLPlaylistFilter caught "+x);
			x.printStackTrace(System.out);
		}
		return next;
	}
	
	
	private void writeTemplatePlaylist( OutputStream out ) throws IOException {
		InputStream in = new BufferedInputStream(new FileInputStream(fTemplateFile));
		BufferedOutputStream bout = new BufferedOutputStream(out);
		PrintStream pout = new PrintStream(bout);
		try{
			int ch;
			while( (ch=in.read()) != -1 ) {
				if( ch != '$' )
					pout.write(ch);
				else {
					ch=in.read();
					if( ch==-1 )
						break;
					else if( ch=='$' )
						pout.write(ch);
					else {
						StringBuffer varBuf = new StringBuffer(10);
						do{
							varBuf.append((char)ch);
						}while( (ch=in.read())!='$' && ch!=-1 );
						String var = varBuf.toString();
						writeVariable(var,pout);
						if( ch==-1 )
							break;
					}
				}
			}
		}finally{
			pout.close();
			in.close();
		}
	}
	
	
	protected void writeVariable( String var, PrintStream out ) {
		// Strip trailing digit into 'index':
		int index = 0;
		int len = var.length();
		char last = var.charAt(len-1);
		if( last>='0' && last<'0'+kHistorySize ) {
			index = last-'0';
			var = var.substring(0,len-1);
		}
		MP3File track = fHistory[index];
		if( track == null )
			return;
		var = var.toLowerCase();
		
		try{
			// Track attributes:
			if( var.equals("title") ) {
				out.print(htmlEscape(track.getTitle()));
			} else if( var.equals("artist") ) {
				out.print(htmlEscape(track.getArtist()));
			} else if( var.equals("album") ) {
				out.print(htmlEscape(track.getAlbum()));
			} else if( var.equals("year") ) {
				String year = htmlEscape(track.getYear());
				if( year != "" )
					out.print("("+year+")");
			} else if( var.equals("time") ) {
				long length = track.getLength();
				long n = length/60;
				if( n < 10 )
					out.print("&nbsp;");
				out.print(n+":");
				n = length%60;
				if( n < 10 )
					out.print("0");
				out.print(n);
			
			// Station or global attributes:
			} else if( var.equals("date") ) {
				out.print( new Date().toString() );			//! Could format more nicely
			} else if( var.equals("station") ) {
				out.print(htmlEscape(fStation.getStationName()));
			} else if( var.equals("host") ) {
				String host = fStation.getProperties().getProperty(Station.kPropHost);
				if( host != null ) {
					String hostURL = fStation.getProperties().getProperty(Station.kPropHostURL);
					if( hostURL != null )
						out.print("<A HREF=\""+hostURL+"\">");
					out.print(htmlEscape(host));
					if( hostURL != null )
						out.print("</A>");
				}
			} else if( var.equals("listeners") ) {
				out.print(fStation.getTransmitterCount());
			} else {
				Radio.WARN(1,"HTMLPlaylistFilter", "Unknown variable $"+var
										+"$ in file "+fTemplateFile);
				out.print("$"+var+"$");
			}
		}catch( IOException x ) {
		}catch( MP3Exception x ) {
		}
	}
				
	
	//FIX: Old method that writes a playlist in a hardcoded format -- should get rid of this
	private void writeHardcodedPlaylist( OutputStream out ) throws IOException, MP3Exception {
		PrintStream pout = new PrintStream(out);
		String title = htmlEscape(fStation.getStationName())+" Playlist";	//FIX: HTML-escape
		pout.print(  "<HTML>\n<HEAD>\n"
					+"\t<TITLE>"+title+"</TITLE>\n"
					+"\t<META HTTP-EQUIV=refresh CONTENT=30>\n"
					+"\t<META HTTP-EQUIV=Pragma  CONTENT=\"no-cache\">\n"
					+"</HEAD>\n\n<BODY>\n"
					+"<H1>"+title+"</H1>\n"
					+"<H3>"+(new Date())+" -- "
					+fStation.getTransmitterCount()+" listeners</H3>\n");

		String host = fStation.getProperties().getProperty(Station.kPropHost);
		if( host != null ) {
			String hostURL = fStation.getProperties().getProperty(Station.kPropHostURL);
			pout.print("<P>Your host: <B>");
			if( hostURL != null )
				pout.print("<A HREF=\""+hostURL+"\">");
			pout.print(htmlEscape(host));
			if( hostURL != null )
				pout.print("</A>");
			pout.print("</B></P>\n");
		}
		
		pout.print("\n<TABLE WIDTH=100%>\n"
				  +"\t<TR>\n"
				  +"\t\t<TD></TD><TD><I>Title</I></TD><TD><I>Artist</I></TD>"
				  +"<TD><I>Album</I></TD><TD><I>Length</I></TD>\n"
				  +"\t</TR>\n");
		for( int i=0; i<kHistorySize; i++ ) {
			MP3File track = fHistory[i];
			if( track != null ) {
				// Print row header / title / artist / album:
				String bg = "\t\t<TD>";
				if( i==1 ) bg += "<B>";
				String end = "</TD>\n";
				if( i==1 ) end = "</B>"+end;
				pout.print("\t<TR BGCOLOR=\""+kRowColor[i]+"\">\n"
						 + bg + "<I>" + kRowTitle[i] + "</I>" + end
						 + bg + "&quot;" + htmlEscape(track.getTitle()) + "&quot;" + end
						 + bg + htmlEscape(track.getArtist()) + end
						 + bg + htmlEscape(track.getAlbum()) );
				// Print year, if present:
				String year = htmlEscape(track.getYear());
				if( year != "" )
					pout.print(" <FONT SIZE=-1>("+year+")</FONT>");
				// Print time (minutes:seconds):
				pout.print(end + bg);
				long length = track.getLength();
				long n = length/60;
				if( n < 10 )
					pout.print("&nbsp;");
				pout.print(n+":");
				n = length%60;
				if( n < 10 )
					pout.print("0");
				pout.print(n + end + "\t</TR>\n");
			}
		}
		pout.print(	"</TABLE>\n\n</BODY>\n</HTML>\n" );
		pout.flush();
	}
	
	
	protected static String htmlEscape( String str ) {
		//FIX: Implement this properly!!
		return str!=null ?str :"";
	}
	
	
	protected MusicIterator fSrc;
	protected Station fStation;
	protected File fHTMLFile;
	protected File fTemplateFile;
	
	protected static final int kHistorySize = 5;
	
	/** Playlist buffer, in reverse order: Element 0 is the next track,
		1 is the current, 2 is the previous, etc. */
	protected MP3File fHistory[] = new MP3File[kHistorySize];
	
	private static final String kRowTitle[] = {"Next:", "Now Playing:", "Previous:", "", ""};
	private static final String kRowColor[] = {"EFEFEF", "FFFF00", "EFEFEF","EFEFEF","EFEFEF"};
	
}
